#pragma once

#include <fstream>
#include <Base_Archiver.h>
#include <Base_Configurator.h>

/**
 * @brief The namespace inaf::oasbo::Archivers contains classes related to file archiving.
 */
namespace inaf::oasbo::Archivers {

/**
 * @brief The FileArchiver class is a derived class of BaseArchiver and provides functionality for archiving packets into files.
 */
class FileArchiver: public BaseArchiver {
protected:
	std::ofstream *myfile = nullptr; /**< Pointer to the output file stream. */
	bool _open; /**< Flag indicating if the file is open. */
	std::string computeFileName(std::string fileName); /**< Computes the file name based on the given file name. */
	std::string resolveEnvVar(std::string path); /**< Resolves environment variables in the given path. */
	FileArchiver(std::string dest); /**< Constructs a FileArchiver object with the given destination. */
	FileArchiver(std::string parent, std::string dest); /**< Constructs a FileArchiver object with the given parent and destination. */
	FileArchiver(); /**< Default constructor for FileArchiver. */
public:
	std::string parentPath; /**< The parent path of the file. */

	int write(Packets::BasePacket&) override; /**< Writes the given packet to the file. */
	int write(Packets::BasePacket&, std::string dest) override; /**< Writes the given packet to the specified destination. */

	int close() override; /**< Closes the file. */
	int open() override; /**< Opens the file. */
	bool is_open() override; /**< Checks if the file is open. */
	void setDest(std::string) override; /**< Sets the file destination of the packets. */
	std::string getDest() override; /**< Gets the destination. */
	~FileArchiver() {
		close();
	}

	friend class FileArchiverBuilder;

};

/**
 * @brief The FileArchiverBuilder class is responsible for building FileArchiver objects.
 */
class FileArchiverBuilder {
protected:
	FileArchiver *arch; /**< Pointer to the FileArchiver object being built. */
public:
	std::string config_target { "filearchiver" }; /**< The configuration target for the FileArchiver. */
	std::string file_name_key { "file_name" }; /**< The key for the file name configuration. */
	std::string archive_path_key { "archive_path" }; /**< The key for the archive path configuration. */

	FileArchiverBuilder(); /**< Default constructor for FileArchiverBuilder. */
	FileArchiverBuilder(std::string dest); /**< Constructs a FileArchiverBuilder object with the given destination. */
	FileArchiverBuilder(std::string parent, std::string dest); /**< Constructs a FileArchiverBuilder object with the given parent and destination. */
	~FileArchiverBuilder();

	void reset(); /**< Resets the FileArchiverBuilder object. */

	FileArchiverBuilder* configFrom(Configurators::BaseConfigurator &conf); /**< Configures the FileArchiverBuilder from the given configurator. */

	FileArchiverBuilder* setParent(std::string parent); /**< Sets the parent path for the FileArchiver. */

	FileArchiverBuilder* setDest(std::string dest); /**< Sets the destination for the FileArchiver. */

	FileArchiver* getArchiver(); /**< Gets the built FileArchiver object. */
};
}
